import React from 'react'
import ReactDOM from 'react-dom'
import deepExtend from 'deep-extend'
import PropTypes from 'prop-types'

let imgRef
let frameNodeRef

class Cropper extends React.Component {
  constructor (props) {
    super(props)
    let {
      originX,
      originY,
      width,
      height,
      selectionNatural,
      fixedRatio,
      allowNewSelection,
      rate,
      styles,
      imageLoaded,
      beforeImageLoaded,
      imageLoadError,
      onDragStop
    } = this.props
    this.state = {
      img_width: '100%',
      img_height: 'auto',
      imgWidth: 200,
      imgHeight: 200,
      imgTop: 0,
      imgLeft: 0,
      originX,
      originY,
      startX: 0,
      startY: 0,
      frameWidth: width,
      fixedRatio,
      selectionNatural,
      allowNewSelection,
      frameHeight: fixedRatio ? width / rate : height,
      dragging: false,
      maxLeft: 0,
      maxTop: 0,
      action: null,
      imgLoaded: false,
      imgBeforeLoaded: false,
      styles: deepExtend({}, defaultStyles, styles),
      imageLoaded,
      beforeImageLoaded,
      imageLoadError,
      onDragStop,
      moved: false,
      originalOriginX: originX,
      originalOriginY: originY,
      originalFrameWidth: width,
      originalFrameHeight: fixedRatio ? width / rate : height
    }
    this.handleDragStart = this.handleDragStart.bind(this)
    this.imgOnLoad = this.imgOnLoad.bind(this)
    this.imgOnError = this.imgOnError.bind(this)
  }

  initStyles () {
    let { originX, originY } = this.props
    const { selectionNatural, img_width, img_height } = this.state

    let frameWidth = this.state.frameWidth || img_width
    let frameHeight = this.state.frameHeight || img_height / 4
    if (selectionNatural) {
      let img = imgRef
      const _rateWidth = img_width / img.naturalWidth
      const _rateHeight = img_height / img.naturalHeight
      const realWidth = Number(frameWidth * _rateWidth)
      const realHeight = Number(frameHeight * _rateHeight)
      const realX = Number(originX * _rateHeight)
      const realY = Number(originY * _rateWidth)

      frameWidth = realWidth
      frameHeight = realHeight
      originX = realX
      originY = realY

      this.setState({
        frameWidth: frameWidth,
        frameHeight: frameHeight,
        originX: originX,
        originY: originY
      })
    }

    const maxLeft = img_width - frameWidth
    const maxTop = img_height - frameHeight

    if (originX + frameWidth >= img_width) {
      originX = img_width - frameWidth
      this.setState({ originX })
    }
    if (originY + frameHeight >= img_height) {
      originY = img_height - frameHeight
      this.setState({ originY })
    }
    this.setState({ maxLeft, maxTop, img_height })
    // calc clone position
    this.calcPosition(frameWidth, frameHeight, originX, originY)
  }
  updateFrame (newWidth, newHeight, newOriginX, newOriginY) {
    this.setState(
      {
        frameWidth: newWidth,
        frameHeight: newHeight,
        originX: newOriginX,
        originY: newOriginY,
        originalFrameWidth: newWidth,
        originalFrameHeight: newHeight,
        originalOriginX: newOriginX,
        originalOriginY: newOriginY
      },
      () => {
        this.initStyles()
      }
    )
  }

  calcPosition (width, height, left, top, move) {
    const { img_width, img_height, fixedRatio } = this.state
    const { rate } = this.props

    if (width < 0 || height < 0) return false
    if (fixedRatio) {
      if (width / img_width > height / img_height) {
        if (width > img_width) {
          width = img_width
          left = 0
          if (fixedRatio) {
            height = width / rate
          }
        }
      } else {
        if (height > img_height) {
          height = img_height
          top = 0
          if (fixedRatio) {
            width = height * rate
          }
        }
      }
    }

    if (width + left > img_width) {
      if (fixedRatio) {
        left = img_width - width
      } else {
        width = width - (width + left - img_width)
      }
    }

    if (height + top > img_height) {
      if (fixedRatio) {
        top = img_height - height
      } else {
        height = height - (height + top - img_height)
      }
    }

    if (left < 0) {
      if (!fixedRatio && !move) {
        width = width + left
      }
      left = 0
    }
    if (top < 0) {
      if (!fixedRatio && !move) {
        height = height + top
      }
      top = 0
    }

    if (width > img_width) {
      width = img_width
    }
    if (height > img_height) {
      height = img_height
    }

    this.setState({
      imgLeft: left,
      imgTop: top,
      imgWidth: width,
      imgHeight: height
    })
  }

  imgOnLoad () {
    if (this.state.imageLoaded) {
      const { imageLoaded } = this.state
      this.setState({ imgLoaded: true })
      imageLoaded()
    }
  }

  imgOnError () {
    if (this.state.imageLoadError) {
      const { imageLoadError } = this.state
      this.setState({ imgLoaded: false })
      imageLoadError({ error: 'Error loading image' })
    }
  }

  imgGetSizeBeforeLoad () {
    var that = this
    setTimeout(function () {
      let img = imgRef
      if (img && img.naturalWidth) {
        const { beforeImageLoaded } = that.state

        var heightRatio = img.offsetWidth / img.naturalWidth
        var widthRatio = img.offsetHeight / img.naturalHeight

        var img_height = Number(img.naturalHeight * heightRatio)
        var img_width = Number(img.naturalWidth * widthRatio)

        let imgSize = that.props.imgSize
        if (imgSize && imgSize.default_width && imgSize.default_height) {
          img_width =
            img.naturalWidth > img.naturalHeight
              ? imgSize.default_width
              : (imgSize.default_width * img.naturalWidth) / img.naturalHeight
          img_height =
            img.naturalHeight > img.naturalWidth
              ? imgSize.default_height
              : (imgSize.default_height * img.naturalHeight) / img.naturalWidth
        }

        that.setState(
          {
            img_height: img_height,
            img_width: img_width,
            imgBeforeLoaded: true
          },
          () => that.initStyles()
        )

        beforeImageLoaded()
      } else if (img) {
        that.imgGetSizeBeforeLoad()
      }
    }, 0)
  }

  createNewFrame (e) {
    if (this.state.dragging) {
      const pageX = e.pageX ? e.pageX : e.targetTouches[0].pageX
      const pageY = e.pageY ? e.pageY : e.targetTouches[0].pageY
      const { rate } = this.props
      const {
        frameWidth,
        frameHeight,
        startX,
        startY,
        offsetLeft,
        offsetTop,
        fixedRatio
      } = this.state

      const _x = pageX - startX
      const _y = pageY - startY

      if (_x > 0) {
        if (_y < 0) {
          return this.calcPosition(
            frameWidth + _x,
            fixedRatio ? (frameWidth + _x) / rate : frameHeight - _y,
            offsetLeft,
            fixedRatio ? offsetTop - _x / rate : offsetTop + _y
          )
        }
        return this.calcPosition(
          frameWidth + _x,
          fixedRatio ? (frameWidth + _x) / rate : frameHeight + _y,
          offsetLeft,
          offsetTop
        )
      }
      if (_y > 0) {
        return this.calcPosition(
          frameWidth - _x,
          fixedRatio ? (frameWidth - _x) / rate : frameHeight + _y,
          offsetLeft + _x,
          offsetTop
        )
      }

      return this.calcPosition(
        frameWidth - _x,
        fixedRatio ? (frameWidth - _x) / rate : frameHeight - _y,
        offsetLeft + _x,
        fixedRatio ? offsetTop + _x / rate : offsetTop + _y
      )
    }
  }

  handleDrag (e) {
    if (this.state.dragging) {
      e.preventDefault()
      let { action } = this.state
      if (!action) return this.createNewFrame(e)
      if (action == 'move') return this.frameMove(e)
      this.frameDotMove(action, e)
    }
  }

  frameMove (e) {
    const {
      originX,
      originY,
      startX,
      startY,
      frameWidth,
      frameHeight,
      maxLeft,
      maxTop
    } = this.state
    const pageX = e.pageX ? e.pageX : e.targetTouches[0].pageX
    const pageY = e.pageY ? e.pageY : e.targetTouches[0].pageY
    let _x = pageX - startX + originX
    let _y = pageY - startY + originY
    if (pageX < 0 || pageY < 0) return false

    if (pageX - startX > 0 || pageY - startY) {
      this.setState({ moved: true })
    }

    if (_x > maxLeft) _x = maxLeft
    if (_y > maxTop) _y = maxTop
    this.calcPosition(frameWidth, frameHeight, _x, _y, true)
  }

  handleDragStart (e) {
    const { allowNewSelection } = this.state
    const action = e.target.getAttribute('data-action')
      ? e.target.getAttribute('data-action')
      : e.target.parentNode.getAttribute('data-action')
    const pageX = e.pageX ? e.pageX : e.targetTouches[0].pageX
    const pageY = e.pageY ? e.pageY : e.targetTouches[0].pageY
    if (action || allowNewSelection) {
      e.preventDefault()
      this.setState({
        startX: pageX,
        startY: pageY,
        dragging: true,
        action
      })
    }
    if (!action && allowNewSelection) {
      let container = this.container
      const { offsetLeft, offsetTop } = container
      this.setState(
        {
          offsetLeft: pageX - offsetLeft,
          offsetTop: pageY - offsetTop,
          frameWidth: 2,
          frameHeight: 2,
          moved: true
        },
        () => {
          this.calcPosition(2, 2, pageX - offsetLeft, pageY - offsetTop)
        }
      )
    }
  }

  handleDragStop (e) {
    if (this.state.dragging) {
      e.preventDefault()
      const frameNode = frameNodeRef
      const { offsetLeft, offsetTop, offsetWidth, offsetHeight } = frameNode
      const { img_width, img_height, onDragStop } = this.state
      this.setState(
        {
          originX: offsetLeft,
          originY: offsetTop,
          dragging: false,
          frameWidth: offsetWidth,
          frameHeight: offsetHeight,
          maxLeft: img_width - offsetWidth,
          maxTop: img_height - offsetHeight,
          action: null
        },
        function () {
          onDragStop(this.values())
        }
      )
    }
  }

  componentDidMount () {
    document.addEventListener('mousemove', this.handleDrag.bind(this))
    document.addEventListener('touchmove', this.handleDrag.bind(this))
    document.addEventListener('mouseup', this.handleDragStop.bind(this))
    document.addEventListener('touchend', this.handleDragStop.bind(this))
    this.imgGetSizeBeforeLoad()
  }
  componentWillUnmount () {
    document.removeEventListener('mousemove', this.handleDrag.bind(this))
    document.removeEventListener('touchmove', this.handleDrag.bind(this))
    document.removeEventListener('mouseup', this.handleDragStop.bind(this))
    document.removeEventListener('touchend', this.handleDragStop.bind(this))
  }
  componentWillReceiveProps (newProps) {
    var width = this.props.width !== newProps.width
    var height = this.props.height !== newProps.height
    var originX = this.props.originX !== newProps.originX
    var originY = this.props.originY !== newProps.originY

    if (width || height || originX || originY) {
      this.updateFrame(
        newProps.width,
        newProps.height,
        newProps.originX,
        newProps.originY
      )
    }
  }
  frameDotMove (dir, e) {
    const pageX = e.pageX ? e.pageX : e.targetTouches[0].pageX
    const pageY = e.pageY ? e.pageY : e.targetTouches[0].pageY
    const { rate } = this.props
    const {
      startX,
      startY,
      originX,
      originY,
      frameWidth,
      frameHeight,
      fixedRatio
    } = this.state

    if (pageY !== 0 && pageX !== 0) {
      const _x = pageX - startX
      const _y = pageY - startY

      if (pageX - startX > 0 || pageY - startY) {
        this.setState({ moved: true })
      }

      let new_width = frameWidth + _x
      let new_height = fixedRatio ? new_width : frameHeight + _y
      switch (dir) {
        case 'ne':
          new_height = frameHeight - _y
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : new_height,
            originX,
            fixedRatio ? originY - _x / rate : originY + _y
          )
        case 'e':
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : frameHeight,
            originX,
            fixedRatio ? originY - (_x / rate) * 0.5 : originY
          )
        case 'se':
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : new_height,
            originX,
            originY
          )
        case 'n':
          new_height = frameHeight - _y
          return this.calcPosition(
            fixedRatio ? new_height * rate : frameWidth,
            new_height,
            fixedRatio ? originX + _y * rate * 0.5 : originX,
            originY + _y
          )
        case 'nw':
          new_width = frameWidth - _x
          new_height = frameHeight - _y
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : new_height,
            originX + _x,
            fixedRatio ? originY + _x / rate : originY + _y
          )
        case 'w':
          new_width = frameWidth - _x
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : frameHeight,
            originX + _x,
            fixedRatio ? originY + (_x / rate) * 0.5 : originY
          )
        case 'sw':
          new_width = frameWidth - _x
          return this.calcPosition(
            new_width,
            fixedRatio ? new_width / rate : new_height,
            originX + _x,
            originY
          )
        case 's':
          new_height = frameHeight + _y
          return this.calcPosition(
            fixedRatio ? new_height * rate : frameWidth,
            new_height,
            fixedRatio ? originX - _y * rate * 0.5 : originX,
            originY
          )
        default:
      }
    }
  }

  crop () {
    const { frameWidth, frameHeight, originX, originY, img_width } = this.state
    let canvas = document.createElement('canvas')
    let img = imgRef
    const _rate = img.naturalWidth / img_width
    const realWidth = frameWidth * _rate
    const realHeight = frameHeight * _rate
    canvas.width = realWidth
    canvas.height = realHeight

    canvas
      .getContext('2d')
      .drawImage(
        img,
        originX * _rate,
        originY * _rate,
        realWidth,
        realHeight,
        0,
        0,
        realWidth,
        realHeight
      )
    return canvas.toDataURL()
  }

  values () {
    const {
      frameWidth,
      frameHeight,
      originX,
      originY,
      img_width,
      img_height,
      selectionNatural,
      moved,
      originalOriginX,
      originalOriginY,
      originalFrameWidth,
      originalFrameHeight
    } = this.state

    let img = imgRef
    let _return = null

    var thisOriginX = moved ? originX : originalOriginX
    var thisOriginY = moved ? originY : originalOriginY
    var thisFrameWidth = moved ? frameWidth : originalFrameWidth
    var thisFrameHeight = moved ? frameHeight : originalFrameHeight

    if (selectionNatural && moved) {
      const _rateWidth = img.naturalWidth / img_width
      const _rateHeight = img.naturalHeight / img_height
      const realWidth = Number(thisFrameWidth * _rateWidth)
      const realHeight = Number(thisFrameHeight * _rateHeight)
      const realX = Number(thisOriginX * _rateHeight)
      const realY = Number(thisOriginY * _rateWidth)
      _return = { width: realWidth, height: realHeight, x: realX, y: realY }
    } else {
      _return = {
        width: thisFrameWidth,
        height: thisFrameHeight,
        x: thisOriginX,
        y: thisOriginY
      }
    }

    return _return
  }

  render () {
    const { dragging, img_height, img_width, imgBeforeLoaded } = this.state
    const { src, disabled } = this.props

    const imageNode = (
      <div
        style={this.state.styles.source}
        ref={inst => (this.sourceNodeRef = inst)}
      >
        <img
          crossOrigin='anonymous'
          src={src}
          style={deepExtend(
            {},
            this.state.styles.img,
            this.state.styles.source_img
          )}
          ref={inst => (imgRef = inst)}
          onLoad={this.imgOnLoad}
          onError={this.imgOnError}
          width={img_width}
          height={img_height}
        />
      </div>
    )

    let disabledStyle = disabled ? { display: 'none', cursor: 'initial' } : {}

    return (
      <div
        ref={inst => (this.containerRef = inst)}
        onMouseDown={disabled ? undefined : this.handleDragStart}
        onTouchStart={disabled ? undefined : this.handleDragStart}
        style={deepExtend({}, this.state.styles.container, {
          position: 'relative',
          height: img_height
        })}
      >
        {imageNode}
        {imgBeforeLoaded ? (
          <div>
            <div style={this.state.styles.modal} />
            <div
              style={deepExtend(
                {},
                this.state.styles.frame,
                dragging ? this.state.styles.dragging_frame : {},
                {
                  display: 'block',
                  left: this.state.imgLeft,
                  top: this.state.imgTop,
                  width: this.state.imgWidth,
                  height: this.state.imgHeight
                }
              )}
              ref={inst => (frameNodeRef = inst)}
            >
              <div style={this.state.styles.clone}>
                <img
                  ref={inst => (this.cloneImgRef = inst)}
                  width={img_width}
                  height={img_height}
                  crossOrigin='anonymous'
                  src={src}
                  style={deepExtend({}, this.state.styles.img, {
                    marginLeft: -this.state.imgLeft,
                    marginTop: -this.state.imgTop
                  })}
                />
              </div>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.move,
                  disabled ? { cursor: 'initial' } : {}
                )}
                data-action='move'
              />
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotCenter,
                  disabledStyle
                )}
                data-action='move'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerCenterVertical
                  )}
                />
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerCenterHorizontal
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotNE
                )}
                data-action='ne'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerNE,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotN
                )}
                data-action='n'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerN,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotNW
                )}
                data-action='nw'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerNW,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotE
                )}
                data-action='e'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerE,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotW
                )}
                data-action='w'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerW,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotSE
                )}
                data-action='se'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerSE,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotS
                )}
                data-action='s'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerS,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.dot,
                  this.state.styles.dotSW
                )}
                data-action='sw'
              >
                <span
                  style={deepExtend(
                    {},
                    this.state.styles.dotInner,
                    this.state.styles.dotInnerSW,
                    disabledStyle
                  )}
                />
              </span>
              <span
                style={deepExtend(
                  {},
                  this.state.styles.line,
                  this.state.styles.lineN,
                  disabledStyle
                )}
                data-action='n'
              />
              <span
                style={deepExtend(
                  {},
                  this.state.styles.line,
                  this.state.styles.lineS,
                  disabledStyle
                )}
                data-action='s'
              />
              <span
                style={deepExtend(
                  {},
                  this.state.styles.line,
                  this.state.styles.lineW,
                  disabledStyle
                )}
                data-action='w'
              />
              <span
                style={deepExtend(
                  {},
                  this.state.styles.line,
                  this.state.styles.lineE,
                  disabledStyle
                )}
                data-action='e'
              />
            </div>
          </div>
        ) : null}
      </div>
    )
  }
}
Cropper.propTypes = {
  src: PropTypes.string.isRequired,
  originX: PropTypes.number,
  originY: PropTypes.number,
  rate: PropTypes.number,
  width: PropTypes.number,
  height: PropTypes.number,
  imgSize: PropTypes.object,
  selectionNatural: PropTypes.bool,
  fixedRatio: PropTypes.bool,
  allowNewSelection: PropTypes.bool,
  disabled: PropTypes.bool,
  styles: PropTypes.object,
  imageLoaded: PropTypes.func,
  beforeImageLoaded: PropTypes.func,
  imageLoadError: PropTypes.func,
  onDragStop: PropTypes.func
}
Cropper.defaultProps = {
  width: 200,
  height: 200,
  selectionNatural: false,
  fixedRatio: true,
  allowNewSelection: true,
  rate: 1,
  originX: 0,
  originY: 0,
  styles: {},
  imageLoaded: function () {},
  beforeImageLoaded: function () {},
  imageLoadError: function () {},
  onDragStop: function () {}
}

var defaultStyles = {
  container: {},
  img: {
    userDrag: 'none',
    userSelect: 'none',
    MozUserSelect: 'none',
    WebkitUserDrag: 'none',
    WebkitUserSelect: 'none',
    WebkitTransform: 'translateZ(0)',
    WebkitPerspective: 1000,
    WebkitBackfaceVisibility: 'hidden'
  },
  clone: {
    width: '100%',
    height: '100%',
    overflow: 'hidden',
    position: 'absolute',
    left: 0,
    top: 0
  },
  frame: {
    position: 'absolute',
    left: 0,
    top: 0,
    bottom: 0,
    right: 0,
    display: 'none'
  },
  dragging_frame: {
    opacity: 0.8
  },
  source: {
    overflow: 'hidden'
  },
  source_img: {
    float: 'left'
  },
  modal: {
    position: 'absolute',
    left: 0,
    top: 0,
    bottom: 0,
    right: 0,
    opacity: 0.4,
    backgroundColor: '#222'
  },
  modal_disabled: {
    opacity: 0
  },
  move: {
    position: 'absolute',
    left: 0,
    top: 0,
    bottom: 0,
    right: 0,
    cursor: 'move',
    outline: '1px dashed #88f',
    backgroundColor: 'transparent'
  },
  dot: {
    zIndex: 10
  },
  dotN: {
    cursor: 'n-resize'
  },
  dotS: {
    cursor: 's-resize'
  },
  dotE: {
    cursor: 'e-resize'
  },
  dotW: {
    cursor: 'w-resize'
  },
  dotNW: {
    cursor: 'nw-resize'
  },
  dotNE: {
    cursor: 'ne-resize'
  },
  dotSW: {
    cursor: 'sw-resize'
  },
  dotSE: {
    cursor: 'se-resize'
  },
  dotCenter: {
    backgroundColor: 'transparent',
    cursor: 'move'
  },
  dotInner: {
    border: '1px solid #88f',
    background: '#fff',
    display: 'block',
    width: 6,
    height: 6,
    padding: 0,
    margin: 0,
    position: 'absolute'
  },
  dotInnerN: {
    top: -4,
    left: '50%',
    marginLeft: -4
  },
  dotInnerS: {
    bottom: -4,
    left: '50%',
    marginLeft: -4
  },
  dotInnerE: {
    right: -4,
    top: '50%',
    marginTop: -4
  },
  dotInnerW: {
    left: -4,
    top: '50%',
    marginTop: -4
  },
  dotInnerNE: {
    top: -4,
    right: -4
  },
  dotInnerSE: {
    bottom: -4,
    right: -4
  },
  dotInnerNW: {
    top: -4,
    left: -4
  },
  dotInnerSW: {
    bottom: -4,
    left: -4
  },
  dotInnerCenterVertical: {
    position: 'absolute',
    border: 'none',
    width: 2,
    height: 8,
    backgroundColor: '#88f',
    top: '50%',
    left: '50%',
    marginLeft: -1,
    marginTop: -4
  },
  dotInnerCenterHorizontal: {
    position: 'absolute',
    border: 'none',
    width: 8,
    height: 2,
    backgroundColor: '#88f',
    top: '50%',
    left: '50%',
    marginLeft: -4,
    marginTop: -1
  },
  fixedRatio: true,
  line: {
    position: 'absolute',
    display: 'block',
    zIndex: 100
  },
  lineS: {
    cursor: 's-resize',
    bottom: 0,
    left: 0,
    width: '100%',
    height: 4,
    background: 'transparent'
  },
  lineN: {
    cursor: 'n-resize',
    top: 0,
    left: 0,
    width: '100%',
    height: 4,
    background: 'transparent'
  },
  lineE: {
    cursor: 'e-resize',
    right: 0,
    top: 0,
    width: 4,
    height: '100%',
    background: 'transparent'
  },
  lineW: {
    cursor: 'w-resize',
    left: 0,
    top: 0,
    width: 4,
    height: '100%',
    background: 'transparent'
  }
}

export default Cropper
